from datetime import datetime
from pathlib import Path
from tkinter.filedialog import askdirectory
import toml
import whisper
import rich
from rich.prompt import Prompt
import rich_click as click
from result import Result, is_ok, is_err, Ok, Err


from local_wisper_model import save_config_to_toml, get_config_from_toml

MODEL = "large"
LOCALPATH = ('/Users/ncdegroot/Library/CloudStorage/'
             'OneDrive-Gedeeldebibliotheken-TilburgUniversity/'
             'Project - Reflective cafe - data')
model = None


def process_file(path):
    output_path = path.with_suffix('.txt')
    try:
        print("Start processing...")
        result = model.transcribe(str(path), verbose=True)
        # false: only progressbar; true: all; no param: no feedback
    except Exception as e:
        print(f"Error while processing {path}: '{e}'. Please fix it")
    else:
        text_to_save = result["text"]
        print(text_to_save)

        # file_name = f"{data_file.split('.')[0]}_{datetime.now().strftime('%Y-%m-%d-%H-%M-%S')}.txt"
        # Open the file in write mode
        with open(output_path, 'w') as file:
            # Write the text to the file
            file.write(text_to_save)

        print(f'Text has been saved to {output_path}')


@click.command()
@click.option("--config",
              default="config.toml",
              help="Specify config file")
@click.option("--init",
              default=False,
              is_flag=True,
              help="Create config.toml file")
def main():
    global model
    # open config, ask for values is needed:
    #  Prompt.ask(msg)
    toml_path = Path("config.toml")
    if not toml_path.exists():
        save_config_to_toml(toml_path, LOCALPATH, MODEL)
    result = get_config_from_toml(toml_path)
    if is_err(result):
        print(f"Exiting due to {result.err}")
        exit(1)
    config = result.ok_value
    if config:
        print(f"Config folder path: {config.folder}")
        print(f"Config model name: {config.model}")
    model = whisper.load_model(config.model)

    # internal_path = pathlib.Path('data')
    teams_path = Path(config.folder)
    path = teams_path

    txt_files = [file for file in path.glob('*') if file.suffix.lower() == '.txt']
    file_stems = [file.stem for file in txt_files]
    # file_stem indicates mp3 has been processed already
    mp3_files = [file for file in path.glob('*') if file.suffix.lower() == '.mp3' and file.stem not in file_stems]

    print(f"{len(mp3_files)} files to be processed")
    for file in mp3_files:
        print(f"Processing {file}")
        process_file(file)


if __name__ == "__main__":

    main()



from click.testing import CliRunner
import transcribetools.local_whisper as commands
"""test needs human interaction as a messagebox and filedialog are presented to the user"""


def test_config_create():
    runner = CliRunner()
    # t = type(run)
    # t = is <class 'rich_click.rich_command.RichCommand'>
    # next line generates warning on 'run' arg 'Expected type 'BaseCommand' but it is 'Any'
    # https://stackoverflow.com/questions/77845322/unexpected-warning-in-click-cli-development-with-python
    # noinspection PyTypeChecker
    # noinspection PyTypeChecker
    result = runner.invoke(commands.cli,
                           ['config', 'create'],
                           input='large\n\n')
    response = result.return_value
    print(f"{response=}" )
    # input='ndegroot\ntheol_credo')
    assert ": large" in result.stdout  # feedback model choice
    assert "(localwhisper.toml)" in result.stdout


# noinspection PyTypeChecker
def test_config_show():
    runner = CliRunner()
    # t = type(run)
    # t = is <class 'rich_click.rich_command.RichCommand'>
    # next line generates warning on 'run' arg 'Expected type 'BaseCommand' but it is 'Any'
    # https://stackoverflow.com/questions/77845322/unexpected-warning-in-click-cli-development-with-python
    result = runner.invoke(commands.cli,
                           ['config', 'show'])
    # no user input
    assert result.exit_code == 0
    assert ": large" in result.stdout  # feedback model choice


# noinspection PyTypeChecker
def test_process():
    runner = CliRunner()
    # t = type(run)
    # t = is <class 'rich_click.rich_command.RichCommand'>
    # next line generates warning on 'run' arg 'Expected type 'BaseCommand' but it is 'Any'
    # https://stackoverflow.com/questions/77845322/unexpected-warning-in-click-cli-development-with-python
    result = runner.invoke(commands.cli,
                           ['process'])
    # no user input
    assert result.exit_code == 0
    assert "string" in result.stdout






from typing import Union, Optional
from pathlib import Path
import click
import toml
from result import is_ok, is_err, Ok, Err, Result
from attrs import define
from rich.console import Console

console = Console(width=120, force_terminal=True)


@define
class Config:
    folder: str
    model: str
    debug: bool = False


def save_config_to_toml(filepath,
                        folder: Union[str, Path] = "",
                        model_name: str = ""):
    try:
        # let's save a clean folder path, so we can use Path() when retrieving
        data = {"folder": str(folder), "model": model_name}
        with open(filepath, 'w') as toml_file:
            # noinspection PyTypeChecker
            toml.dump(data, toml_file)
    except Exception as e:
        click.secho(f"Error saving config to "
                    f"TOML file @ {filepath}: {e}", fg='red')


def get_config_from_toml(filepath) -> Result:
    try:
        with open(filepath, 'r') as toml_file:
            data = toml.load(toml_file)
    except FileNotFoundError:
        Err("TOML file not found.")
    except toml.TomlDecodeError:
        Err("Error decoding TOML file.")
    except Exception as e:
        Err(f"Unexpected error: {e}")
    else:
        config = Config(**data)  # as data is flat, it's ok
        return Ok(config)


def ask_choice(msg: str, choices) -> int:

    # Print het keuzemenu
    console.print(
            f"[bold magenta]{msg}[/bold magenta]\n"
            "[yellow]Kies een van de volgende opties:[/yellow]"
    )
    for i, choice in enumerate(choices, start=1):
        console.print(f"{i}. {choice}")

    # Vraag input van de gebruiker
    user_input = Prompt.ask(
        "Voer het nummer van je keuze in",
        choices=[str(i) for i in range(1, len(choices) + 1)]
    )

    # Verwerk de keuze
    chosen_option = choices[int(user_input) - 1]  # Converteer input naar index
    console.print(f"[green]Je hebt gekozen voor:[/green] {chosen_option}")

    return chosen_option


def show_config(result: Result):
    if is_err(result):
        click.echo(f"Exiting due to {result.err}")
        return False
    config = result.ok_value
    if config:
        click.echo(f"Config folder path: {config.folder}")
        click.echo(f"Config model name: {config.model}")
    return True



import time
from pathlib import Path
# import tkinter as tk
from tkinter.filedialog import askdirectory
# from tkinter import messagebox as mb
import pymsgbox as msgbox
# import toml
import whisper
# import rich
from rich.prompt import Prompt
import rich_click as click
from result import Result, is_ok, is_err, Ok, Err  # noqa: F401
import soundfile as sf
from .local_wisper_model import save_config_to_toml, get_config_from_toml, show_config, console

# logging.getLogger("python3").setLevel(logging.ERROR)
# loggers = [logging.getLogger(name) for name in logging.root.manager.loggerDict]
# tk bug in sequoia
# import sys
# sys.stderr = open("log", "w", buffering=1)
# can't find the 'python3' logger to silence

MODEL = "large"
# LOCALPATH = ('/Users/ncdegroot/Library/CloudStorage/'
#              'OneDrive-Gedeeldebibliotheken-TilburgUniversity/'
#              'Project - Reflective cafe - data')
LOCALPATH = Path.cwd()
model = None


def process_file(path, args):
    output_path = path.with_suffix('.txt')
    try:
        click.echo("Start processing...")
        result = model.transcribe(str(path),
                                  verbose=True,
                                  **args,)
        # false: only progressbar; true: all; no param: no feedback
    except Exception as e:
        click.echo(f"Error while processing {path}: '{e}'. Please fix it")
    else:
        text_to_save = result["text"]
        click.echo(text_to_save)

        # file_name = f"{data_file.split('.')[0]}_{datetime.now().strftime('%Y-%m-%d-%H-%M-%S')}.txt"
        # file_name = output_path
        # Open the file in write mode
        with open(output_path, 'w') as file:
            # Write the text to the file
            file.write(text_to_save)

        click.echo(f'Text has been saved to {output_path}')


@click.group(no_args_is_help=True,
             epilog='Use config --help or process --help to see options.\nCheck out the docs at https://gitlab.uvt.nl/tst-research/transcribetools for more details')
@click.version_option(package_name='transcribetools')
@click.option('--debug/--no-debug',
              '-d/-n',
              help="Print debug messages and timing information",
              default=False)
@click.pass_context  # our 'global' context
@click.option("--configfilename",
              '-c',
              default=Path.home()/"localwhisper.toml",
              help="Specify config file to use",
              # show_default=True,
              metavar="FILE",
              type=click.Path(exists=True, dir_okay=False, readable=True, resolve_path=True),
              show_choices=False,
              required=False,
              #prompt="Enter new config filename or accept default",
              )
def cli(ctx: click.Context, debug, configfilename):
    global model
    # open config, ask for values if needed:
    #  Prompt.ask(msg)
    home = Path.home()
    config_path = home / configfilename
    if debug:
        click.echo(f"Config_path: {config_path}")
    if not config_path.exists():
        save_config_to_toml(config_path, LOCALPATH, MODEL)
    result = get_config_from_toml(config_path)  # has the default values (homedir, large)
    if is_err(result):
        click.echo(f"Exiting due to {result.err}")
        exit(1)
    config = result.ok_value
    if config:
        # click.echo("Config")
        click.echo(f"Config filename: {config_path}")
        # click.echo(f"Folder path for soundfiles: {config.folder}")
        # click.echo(f"Transcription model name: {config.model}")
    config.debug = debug
    ctx.obj = config
    # process_files(config)


# the `cli` subcommand 'process'

@cli.command("process", help="Using current configuration, transcribe all soundfiles in the folder")
@click.option('--language',
              '-l',
              default="AUTO",
              type=click.Choice(['NL', 'UK', 'AUTO'],
                                case_sensitive=False,
                                ))
@click.option('--prompt',
              '-p',
              help='-p "" You can add special prompts and words (spelling) (max 224 chars) see '
                   'https://cookbook.openai.com/examples/whisper_prompting_guide')
@click.pass_obj  # in casu the config obj
def process(config, prompt, language):
    global model
    # config = config
    if config.debug:
        click.echo(f"Load model: {config.model}")
    model = whisper.load_model(config.model)
    soundfiles_path = Path(config.folder)
    if config.debug:
        click.echo(f"Folder path for soundfiles: {soundfiles_path}")
        click.echo(f"{language=}, {prompt=} ")

    txt_files = [file for file in soundfiles_path.glob('*') if file.suffix.lower() == '.txt']
    file_stems = [file.stem for file in txt_files]
    # a txt file_stem indicates mp3 has been processed already
    soundfiles = [file for file in soundfiles_path.glob('*') if file.suffix.lower()
                  in '.mp3mp3.mp4.mpweg.mpga.m4a.wav.webm' and file.stem not in file_stems]
    click.echo(f"{len(soundfiles)} files to be processed")
    duration = 0
    start = time.perf_counter()

    for file in soundfiles:
        f, samplerate = sf.read(file)
        duration += len(f) / samplerate
        click.echo(f"Processing {file}")
        args = dict()
        if language != "AUTO":
            args["language"] = language
        if prompt:
            args["prompt"] = prompt
        process_file(file, args)
    if soundfiles:
        process_time = time.perf_counter() - start
        click.echo(f"Total sound duration: {duration:.1f} seconds, \n"
                   f"processing time: {process_time:.1f} seconds, \n"
                   f"realtime factor: {(process_time/duration):.2f}")


# the `cli` command config
@cli.group("config", help="configuration")
def config():
    pass


# the `config` create subcommand
@click.command("create", help="Create new configuration file")
def create():
    msg = "Select folder to containing the sound files"
    click.echo(msg)
    # root = tk.Tk()
    # root.focus_force()
    # Cause the root window to disappear milliseconds after calling the filedialog.
    # root.after(100, root.withdraw)
    # tk.Tk().withdraw()
    # hangs: mb.showinfo("msg","Select folder containing the sound files")
    msgbox.alert(msg, "info")
    # "title" only supported on linux ith wv ...
    folder = askdirectory(title="Select folder to monitor containing the sound files",
                          mustexist=True,
                          initialdir='~')
    choices = ["tiny", "base", "small", "medium", "large", "turbo"]
    # inx = ask_choice("Choose a model", choices)
    # model = choices[inx]
    model = Prompt.ask("Choose a model",
                       console=console,
                       choices=choices,
                       show_default=True,
                       default="large")
    config_name = Prompt.ask("Enter a name for the configuration file",
                             show_default=True,
                             default="localwhisper.toml")
    config_path = Path(config_name)
    toml_path = config_path.with_suffix(".toml")
    while toml_path.exists():  # current dir
        result = get_config_from_toml(toml_path)
        click.secho("Already exists...", fg='red')
        show_config(result)
        overwrite = Prompt.ask("Overwrite?",
                               choices=["y", "n"],
                               default="n",
                               show_default=True)
        if overwrite == "y":
            break
        else:
            return
    # Prompt.ask("Enter model name")
    save_config_to_toml(toml_path, folder, model)
    click.echo(f"{toml_path} saved")


# the 'config' show subcommand
@click.command("show", help="Show current configuration file")
@click.pass_obj
def show(config):
    click.echo(f"Config folder path: {config.folder}")
    click.echo(f"Config model name: {config.model}")


# connect the subcommand to `config'
config.add_command(create)
config.add_command(show)

if __name__ == "__main__":
    cli()



"""uses cloud version of whisper model to transcribe
a key-like project name has tob be created in de OpenAi site,
you need an OpenAI account for that"""
from openai import OpenAI
from tiutools import OpenAICloudConfig

# 08-01-2025 laatste run: $0.02

config = OpenAICloudConfig(reset_keys=False)  # the keys are safe in the macOS keychain

api_key = config.api_key
project = config.prj_id

client = OpenAI(api_key=api_key,
                project=project)

audio_file = open("data/fragment.mp3", "rb")

transcription = client.audio.transcriptions.create(
  model="whisper-1",
  file=audio_file,
  language="nl"
)
print(transcription.text)


"""/Users/ncdegroot/workspace/TranscribeTools/.venv/bin/python3.12 /Users/ncdegroot/workspace/TranscribeTools/cloud_openai.py 
 Oké, we wachten op iemand anders. Ja, Margot Mertz komt. Oké. Ze is van het leger des _hels_. Aha. En iemand van 
 vluchtelingen in de knel. Dus ik kijk of ik haar contact heb. Oké. Ja, prima. Ja. We gaan beginnen. Dus toch internet? 
 Nee, nee, nee. Oké. Oké. Ik ga proberen om rustig te praten. Want we hebben hier twee gasten uit Rome. 
 Anita? Yes. Yes. From Zagreb. Oh, Zagreb. Zagreb. Yes. Oké. En Annette uit Duitsland. Echt goed, ja. De andere 
 deelnemers zijn allemaal uit Nederland. Nederland en Lidia werken hier bij het NSV. En dit zit ook bij het gesprek. 
 Wij hebben ongeveer tot half één dit gesprek met elkaar. Daarna... Twaalf uur. Twaalf uur? Ja, tot twaalf uur. 
 Twaalf uur. Daarna gaan we lekker met elkaar lunchen. Oh. Nee, we gaan op lunch starten. We starten tot half één, ja. 
 Ja, klopt. Dan moet ik straks even praten met Mathijs. We hebben nog een pauze, denk ik, hè? We hebben nog een pauze, 
 ja. Oké. Ja. We hebben nog een pauze. En ik ga denk ik de deur even dichtdoen. Ja? Dan hebben we even... Wil jij dat 
 ik doe? Ja. Maar misschien is het leuk om even van elkaar te horen wie hier allemaal zitten. Van welke organisatie 
 je bent. Wat voor werk je doet. Wie mag ik het woord geven? Zal ik beginnen? Mijn naam is Christel. Mijn naamstikker 
 staat op mijn jasje. Christel Teekman. En ik ben de, ja, hoe zeg je dat? Promovenda van Kees. Ik ga de komende jaren 
 hier bij MST onderzoek doen. En daar ben ik net mee begonnen. Echt 1 september. Je valt vandaag met je neus in de doek. 
 Dit is voor mij ook de allereerste keer. Of de tweede keer dat ik hier ben. Maar in ieder geval met deze groep mensen. 
 En ik werk hiernaast ook nog op Hogeschool FIA. Dat is een hogeschool in Zwolle. En daar woon ik ook. En ik werk 3 
 dagen hier dan. Aan de Tilburg University. En Kees en MST en jullie groep heeft zeg maar een heel mooi onderzoek 
 geschreven de laatste jaren. En daar mag ik, ja, als promovenda de komende 4 jaren mee bezighouden. Mijn achtergrond 
 is, ik ben ook een social worker. Ik heb in de vrouwenhulpverlening gewerkt. In _relaxering_ gewerkt.
"""